import unittest

from emoji_data.sequence import EmojiSequence
from emoji_data.utils import reload_test_data


class SequenceTestCase(unittest.TestCase):
    test_data = []

    @classmethod
    def setUpClass(cls):
        cls.test_data = reload_test_data()
        EmojiSequence.initial()

    def test_code_points(self):
        for code_points, qualified_type in self.test_data:
            if qualified_type == 'fully-qualified':
                EmojiSequence.from_codes(code_points)


if __name__ == '__main__':
    unittest.main()



import os
from enum import Enum
from typing import Union, List, Iterable

from pkg_resources import Requirement, resource_stream
from .utils import BaseDictContainer, preproc_line_data

from . import version

__all__ = ['EmojiCharProperty', 'EmojiCharacter']

PACKAGE = '.'.join(version.__name__.split('.')[:-1])
DATAFILE_STREAM = resource_stream(
    Requirement.parse(PACKAGE),
    os.path.join(*(PACKAGE.split('.') + ['data', 'emoji-data.txt']))
)

IGNORE_CODES = [
    0x0023,  # 1.1  [1] (#️)       number sign
    0x002A,  # 1.1  [1] (*️)       asterisk
] + list(range(0x0030, 0x0039 + 1))  # 1.1 [10] (0️..9️)    digit zero..digit nine


class EmojiCharProperty(Enum):
    """Emoji Character Properties

    see: http://www.unicode.org/reports/tr51/#Emoji_Properties
    """
    EMOJI = 'Emoji'
    EPRES = 'Emoji_Presentation'
    EMOD = 'Emoji_Modifier'
    EBASE = 'Emoji_Modifier_Base'
    ECOMP = 'Emoji_Component'
    EXTPICT = 'Extended_Pictographic'


class _MetaClass(BaseDictContainer):
    pass


class EmojiCharacter(metaclass=_MetaClass):
    """emoji character — A character that has the Emoji property. These characters are recommended for use as emoji.

    see: http://www.unicode.org/reports/tr51/#Emoji_Characters
    """

    def __init__(self, code: int, properties: Union[List[EmojiCharProperty], EmojiCharProperty] = None):
        self._code = code
        if code > 0xffff:
            self._regex = r'\U{:08X}'.format(code)
        else:
            self._regex = r'\u{:04X}'.format(code)
        self._properties = list()  # type: List[EmojiCharProperty]
        if properties is not None:
            if isinstance(properties, Iterable):
                self._properties = list(properties)
            else:
                self._properties = [properties]

    def __str__(self):
        return self.string

    def __repr__(self):
        return '<{} hex={} char={!r}>'.format(
            type(self).__name__,
            self.hex,
            self.string,
        )

    _initial = False

    @classmethod
    def initial(cls):
        """Initial the class

        Load Emoji Characters and there properties from package data file into class internal dictionary

        .. note:: **MUST** call this before other operations on the class
        """
        if cls._initial:
            return
        for data in DATAFILE_STREAM:  # type: bytes
            line = preproc_line_data(data)
            if not line:
                continue
            code_points_text, property_text = (part.strip() for part in line.split(';', 1))
            code_points_parts = code_points_text.split('..', 1)
            property_ = EmojiCharProperty(property_text)
            for code in range(int(code_points_parts[0], 16), 1 + int(code_points_parts[-1], 16)):
                try:
                    inst = cls[code]  # type: EmojiCharacter
                except KeyError:
                    cls[code] = cls(code, property_)  # type: EmojiCharacter
                else:
                    inst.add_property(property_)
        cls._initial = True

    @classmethod
    def is_emoji_char(cls, c: str) -> bool:
        """Check if emoji character

        :param str c: character to check
        :return: Whether if emoji character
        :rtype: bool
        """
        if len(c) != 1:
            raise ValueError('Length of char string should be 1')
        code = ord(c)
        return code not in IGNORE_CODES and code in cls

    @classmethod
    def is_emoji_code(cls, code: int) -> bool:
        """Check if an unicode integer value emoji character

        :param int code: code to check
        :return: Whether if emoji character
        :rtype: bool
        """
        return code not in IGNORE_CODES and code in cls

    @classmethod
    def is_emoji_hex(cls, hex_str: str) -> bool:
        """Check if an unicode integer hex express emoji character

        :param str hex_str: hex code to check
        :return: Whether if emoji character
        :rtype: bool
        """
        code = int(hex_str, 16)
        return code not in IGNORE_CODES and code in cls

    def add_property(self, val: EmojiCharProperty):
        if val not in self._properties:
            self._properties.append(val)

    @property
    def code(self) -> int:
        """Unicode integer value of the Emoji

        :type: int
        """
        return self._code

    @property
    def properties(self) -> List[EmojiCharProperty]:
        """Property description text of the Emoji

        :type: List[EmojiCharProperty]
        """
        return self._properties

    @property
    def regex(self) -> str:
        """Regular express for the Emoji

        :type: str
        """
        return self._regex

    @property
    def hex(self) -> str:
        """Hex style text of the Emoji's Unicode

        :type: str
        """
        return hex(self._code)

    @property
    def string(self) -> str:
        """Emoji character string

        :type: string
        """
        return chr(self._code)

    @classmethod
    def from_code(cls, val):  # type: (int)->EmojiCharacter
        """Get an :class:`EmojiCharacter` instance by Emoji Unicode's integer value

        :param int val: Integer value of the Emoji's Unicode
        :return: Instance returned from the class's internal dictionary
        :rtype: EmojiCharacter
        :raises KeyError: When integer code not found in the class' internal dictionary
        """
        return cls[val]

    @classmethod
    def from_char(cls, val):  # type: (str)->EmojiCharacter
        """Get an :class:`EmojiCharacter` instance by Emoji Unicode character

        :param val: Emoji character
        :return: Instance returned from the class's internal dictionary
        :rtype: EmojiCharacter
        :raises KeyError: When character not found in the class' internal dictionary
        """
        return cls[ord(val)]

    @classmethod
    def from_hex(cls, val):  # type: (str)->EmojiCharacter
        """Get an :class:`EmojiCharacter` instance by Emoji Unicode's HEX string

        :param val: Emoji Unicode's HEX
        :return: Instance returned from the class's internal dictionary
        :rtype: EmojiCharacter
        :raises KeyError: When code not found in the class' internal dictionary
        """
        return cls[int(val, 16)]



import os

from pkg_resources import Requirement, resource_stream

from emoji_data import version

PACKAGE = '.'.join(version.__name__.split('.')[:-1])
TEST_DATAFILE_STREAM = resource_stream(
    Requirement.parse(PACKAGE),
    os.path.join(*(PACKAGE.split('.') + ['data', 'emoji-test.txt']))
)


def reload_test_data():
    result = []
    TEST_DATAFILE_STREAM.seek(0)
    for line in TEST_DATAFILE_STREAM:
        line = line.strip()
        if not line:
            continue
        line = line.decode('utf-8')
        if line[0] in ('#', ';'):
            continue
        line = line.split('#', 1)[0].strip()
        code_points, qualified_type = (part.strip() for part in line.split(';', 1))
        result.append((
            [int(s, 16) for s in code_points.split()],
            qualified_type
        ))
    return result


def preproc_line_data(data: bytes) -> str:
    data = data.strip()
    if not data:
        return ''
    if data[0] in b'#;':
        return ''
    return data.decode('utf-8').split('#', 1)[0].strip()


class BaseDictContainer(type):
    def __new__(mcs, name, bases, attrs):  # pylint:disable=bad-mcs-classmethod-argument
        mcs._data = dict()
        return super().__new__(mcs, name, bases, attrs)

    def __setitem__(self, key, value):  # pylint: disable=C0203
        self._data[key] = value

    def __delitem__(self, key):  # pylint: disable=C0203
        del self._data[key]

    def __getitem__(self, key):  # pylint: disable=C0203
        return self._data[key]

    def __contains__(self, key):  # pylint: disable=C0203
        return key in self._data

    def __iter__(self):  # pylint: disable=bad-mcs-method-argument
        for k, v in self._data.items():  # pylint: disable=invalid-name
            yield k, v

    def __len__(self):  # pylint: disable=C0203
        return len(self._data)



"""
A library represents emoji sequences and characters from the data files listed in `Unicode® Technical Standard #51 <http://www.unicode.org/reports/tr51/>`_
"""
from .character import EmojiCharacter, EmojiCharProperty
from .sequence import EmojiSequence
from .version import version as __version__



import os
import re
from typing import Union, List, Iterable

from pkg_resources import Requirement, resource_stream

from . import version
from .character import EmojiCharacter
from .utils import BaseDictContainer, preproc_line_data

__all__ = ['EmojiSequence']

PACKAGE = '.'.join(version.__name__.split('.')[:-1])
SEQUENCES_DATAFILE_STREAM = resource_stream(
    Requirement.parse(PACKAGE),
    os.path.join(*(PACKAGE.split('.') + ['data', 'emoji-sequences.txt']))
)
ZWJ_SEQUENCES_DATAFILE_STREAM = resource_stream(
    Requirement.parse(PACKAGE),
    os.path.join(*(PACKAGE.split('.') + ['data', 'emoji-zwj-sequences.txt']))
)


class _MetaClass(BaseDictContainer):
    pass


class EmojiSequence(metaclass=_MetaClass):
    """Emoji and Text Presentation Sequences used to represent emoji

    see: http://www.unicode.org/reports/tr51/#Emoji_Variation_Sequences
    """

    def __init__(self, chars: Union[Iterable[EmojiCharacter], EmojiCharacter], type_: str = '', desc: str = ''):
        if isinstance(chars, Iterable):
            self._chars = list(chars)
        else:
            self._chars = [chars]
        self._type = type_.strip()
        self._desc = desc.strip()
        #
        self._codes = [m.code for m in self._chars]
        self._string = ''.join(m.string for m in self._chars)
        self._regex = ''.join(m.regex for m in self._chars)
        self._regex_compiled = re.compile(self._regex)

    def __str__(self):
        return self._string

    def __repr__(self):
        return '<{} codes={} text={!r}>'.format(
            type(self).__name__,
            self._codes,
            self._string,
        )

    _initialed = False
    pattern = None
    """Compiled regular express pattern object for all-together Emoji sequences.
    """

    @classmethod
    def initial(cls):
        """Initial the class

        Load Emoji Characters and there properties, the sequences from package data file into class internal dictionary

        .. note:: **MUST** call this before other operations on the class
        """
        if cls._initialed:
            return
        EmojiCharacter.initial()
        for fp in SEQUENCES_DATAFILE_STREAM, ZWJ_SEQUENCES_DATAFILE_STREAM:
            for data in fp:  # type: bytes
                line = preproc_line_data(data)
                if not line:
                    continue
                code_points, type_field, description = (part.strip() for part in line.split(';', 2))
                # codes ...
                code_points_parts = code_points.split('..', 1)  # begin..end form
                if len(code_points_parts) > 1:
                    for code in range(int(code_points_parts[0], 16), 1 + int(code_points_parts[1], 16)):
                        inst = cls(EmojiCharacter.from_code(code), type_field, description)
                        cls[inst.string] = inst
                else:
                    chars = (EmojiCharacter.from_code(code) for code in (int(s, 16) for s in code_points.split()))
                    inst = cls(chars, type_field, description)
                    cls[inst.string] = inst
        # build regex
        seqs = sorted((m for _, m in cls), key=lambda x: len(x.codes), reverse=True)  # type: List[EmojiSequence]
        exp = r'|'.join(m.regex for m in seqs)
        pat = re.compile(exp)
        cls.pattern = pat
        # initialed OK
        cls._initialed = True

    @classmethod
    def from_text(cls, text):  # type: (str)->EmojiSequence
        """Get an :class:`EmojiSequence` instance by text

        :param text: Pass-in text
        :return: Instance returned from the class's internal dictionary
        :rtype: EmojiSequence
        :raises RuntimeError: When non-emoji character in text
        :raises KeyError: When passed-in value not found in the class' internal dictionary
        """
        text = text.strip()
        if not all(ord(s) in EmojiCharacter for s in text):
            raise RuntimeError('Not all characters in the text is Emoji character.')
        try:
            return cls[text]
        except KeyError:
            raise KeyError('[{}]({!r})'.format(' '.join(hex(ord(c)).upper() for c in text), text))

    @classmethod
    def from_characters(cls, characters):  # type: (Iterable[EmojiCharacter])->EmojiSequence
        """Get an :class:`EmojiSequence` instance by :class:`EmojiCharacter` object or list

        :param Iterable[EmojiCharacter] characters: Pass-in object or list/iterable
        :return: Instance returned from the class's internal dictionary
        :rtype: EmojiSequence
        :raises KeyError: When passed-in value not found in the class' internal dictionary
        """
        text = ''.join(m.string for m in characters)
        return cls.from_text(text)

    @classmethod
    def from_codes(cls, codes):  # type: (Iterable[int])->EmojiSequence
        """Get an :class:`EmojiSequence` instance by a list of unicode integer value

        :param Iterable[int] codes: Pass-in int list
        :return: Instance returned from the class's internal dictionary
        :rtype: EmojiSequence
        :raises KeyError: When passed-in value not found in the class' internal dictionary
        """
        text = ''.join(EmojiCharacter.from_code(m).string for m in codes)
        return cls.from_text(text)

    @classmethod
    def from_hexes(cls, hexes):  # type: (Iterable[str])->EmojiSequence
        """Get an :class:`EmojiSequence` instance by a list of unicode hex express

        :param Iterable[str] hexes: Pass-in int list
        :return: Instance returned from the class's internal dictionary
        :rtype: EmojiSequence
        :raises KeyError: When passed-in value not found in the class' internal dictionary
        """
        text = ''.join(EmojiCharacter.from_hex(m).string for m in hexes)
        return cls.from_text(text)

    @property
    def chars(self) -> List[EmojiCharacter]:
        """Emoji character objects list which makes up the Emoji Sequence

        :type: List[EmojiCharacter]
        """
        return self._chars

    @property
    def string(self) -> str:
        """string of the Emoji Sequence

        :type: str
        """
        return self._string

    @property
    def regex(self) -> str:
        """Regular express of the Emoji Sequence

        :type: str
        """
        return self._regex

    @property
    def regex_compiled(self):
        """Compiled regular express pattern of the Emoji Sequence
        """
        return self._regex_compiled

    @property
    def codes(self) -> List[int]:
        """List of unicode integer value of the characters who make up Emoji Sequence

        :type: List[int]
        """
        return self._codes



import unittest

from emoji_data.character import EmojiCharacter
from emoji_data.utils import reload_test_data


class CharacterTestCase(unittest.TestCase):
    test_data = []

    @classmethod
    def setUpClass(cls):
        cls.test_data = reload_test_data()
        EmojiCharacter.initial()

    def test_code_points(self):
        for code_points, _ in self.test_data:
            for code in code_points:
                self.assertTrue(code in EmojiCharacter)


if __name__ == '__main__':
    unittest.main()



import os
import unittest

from emoji_data.sequence import EmojiSequence


class SequenceRegexTestCase(unittest.TestCase):
    test_data = []

    @classmethod
    def setUpClass(cls):
        EmojiSequence.initial()

    def test_no_emoji(self):
        for s in (
                '',
                ' ',
                '\n',
                'abc',
                ' abc\n bcd'
        ):
            self.assertIsNone(EmojiSequence.pattern.match(s))

    def test_single_1char_emoji(self):
        s = '😀'
        self.assertIsNotNone(EmojiSequence.pattern.match(s))

    def test_1char_emoji_started_text(self):
        s = '😀笑脸'
        self.assertIsNotNone(EmojiSequence.pattern.match(s))
        self.assertIsNotNone(EmojiSequence.pattern.search(s))

    def test_1char_emoji_in_text(self):
        s = '这是😀笑脸'
        self.assertIsNone(EmojiSequence.pattern.match(s))
        m = EmojiSequence.pattern.search(s)
        self.assertIsNotNone(m)
        self.assertEqual(m.span(), (2, 3))

    def test_many_1char_emoji_in_text(self):
        s = '1😛 2😛 3😛'
        cnt = 0
        m = EmojiSequence.pattern.search(s)
        while m:
            cnt += 1
            m = EmojiSequence.pattern.search(s, m.end())
        self.assertEqual(cnt, 3)

    def test_sigle_multichar_emoji(self):
        s = '☺️'
        self.assertEqual(len(s), 2)
        self.assertIsNotNone(EmojiSequence.pattern.match(s))

    def test_sigle_multichar_emoji_started_text(self):
        s = '☺️ 也是笑脸'
        self.assertIsNotNone(EmojiSequence.pattern.match(s))

    def test_sigle_multichar_emoji_in_text(self):
        s = '这个☺️也是笑脸'
        self.assertIsNone(EmojiSequence.pattern.match(s))
        m = EmojiSequence.pattern.search(s)
        self.assertIsNotNone(m)
        self.assertEqual(m.span(), (2, 4))

    def test_many_multichar_emoji_in_text(self):
        s = '1☺️ 2☺️ 3☺️'
        cnt = 0
        m = EmojiSequence.pattern.search(s)
        while m:
            cnt += 1
            m = EmojiSequence.pattern.search(s, m.end())
        self.assertEqual(cnt, 3)

    def test_many_multichar_emoji_in_multiline_text(self):
        s = '1☺️ {0}2☺️ {0}3☺️'.format(os.linesep)
        cnt = 0
        m = EmojiSequence.pattern.search(s)
        while m:
            cnt += 1
            m = EmojiSequence.pattern.search(s, m.end())
        self.assertEqual(cnt, 3)


if __name__ == '__main__':
    unittest.main()
